SystemOrganization addCategory: #'Moose-MonticelloImporter'!

TestCase subclass: #MooseMonticelloImporterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

!MooseMonticelloImporterTest methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/4/2009 16:25'!
testBasic
	| importer mooseModel noStubClasses allClasses objectClass testClass classWithIncomingAcess |
	importer := MooseMonticelloCacheImporter new.
	mooseModel := importer importLastVersionOf: 'Moose-MonticelloImporter'.

	allClasses := (mooseModel allClasses).
	self assert: (mooseModel class == MooseModel).
	noStubClasses := (allClasses reject: #isStub).
	self assert: noStubClasses size > 5.
	self assert: (allClasses size > 30).
	

	testClass := allClasses entityNamed: self class name.
	self assert: (testClass methods size > 0).
	self assert: (testClass methods 
						allSatisfy: [:each | each parentType == testClass]).
	"number of lines of code"
	self assert: (testClass numberOfLinesOfCode > 20).



	objectClass := allClasses entityNamed: #Object.
	self assert: (objectClass isStub).
	self assert: (objectClass methods isEmpty).
	
	self assert: (mooseModel allMethods allSatisfy: [:each | each mooseModel == mooseModel ]).
	
	"incoming access group"
	classWithIncomingAcess := mooseModel allClasses detect: [:cls | cls incomingAccessesGroup notEmpty] ifNone: nil.
	self assert: classWithIncomingAcess notNil.
	
! !

AbstractImporter subclass: #MooseMonticelloImporter
	instanceVariableNames: 'repositoryCache classes scope globals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

MooseMonticelloImporter subclass: #MooseMonticelloCacheImporter
	instanceVariableNames: 'directoryString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

!MooseMonticelloCacheImporter methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/1/2009 09:43'!
setRepository
	repositoryCache := MCCacheRepository default! !

MooseMonticelloImporter subclass: #MooseMonticelloHTTPImporter
	instanceVariableNames: 'httpUrl'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

!MooseMonticelloHTTPImporter class methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:50'!
onURL: anHTTPURL
	"
	(self new onURL: 'http://www.squeaksource.com/PersonalSetting') importFileNamed: 'PersonalSetting-simon_denier.16.mcz' 
	"
	^ (self onURL: anHTTPURL) import! !

!MooseMonticelloHTTPImporter methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:35'!
onURL: url
	httpUrl := url! !

!MooseMonticelloHTTPImporter methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/1/2009 09:43'!
setRepository
	repositoryCache := MCHttpRepository 
		location: httpUrl
		user: ''
		password: ''! !

!MooseMonticelloImporter methodsFor: 'populating' stamp: 'tg 9/20/2009 18:10'!
createClassNamed: classNameSymbol ivs: varNameStrings superclassName: superclassName
	"The superclass is not set here. This is done later"
	| famixClass |
	famixClass := FAMIXClass new name: classNameSymbol asSymbol.
	varNameStrings do: [:varName | 
		famixClass addAttribute: (FAMIXAttribute new name: varName asSymbol)].
	self inModel add: famixClass.
	famixClass propertyNamed: #superclassName put: superclassName.						
	^ famixClass! !

!MooseMonticelloImporter methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 21:03'!
createGlobalVariable: name value: value 
	 
	| global | 
	global := self globals 
		at: name 
		put: FAMIXGlobalVariable new. 
	global name: name. 
	Smalltalk at: name ifPresent: [:each | global declaredType: each class mooseName].
	^global! !

!MooseMonticelloImporter methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 20:48'!
createUnknownVariable: name

	| unknown |
	unknown := globals
		at: name
		put: FAMIXUnknownVariable new.
	unknown name: name.
	^unknown! !

!MooseMonticelloImporter methodsFor: 'populating' stamp: 'Alexandre.Bergel 5/1/2009 12:00'!
ensureClassNamed: classNameSymbol
	| cls |
	cls := self inModel allClasses detect: [:each | each name == classNameSymbol] ifNone: [nil].
	cls ifNotNil: [ ^ cls ].
	
	cls := self createClassNamed: classNameSymbol ivs: #() superclassName: nil.
	cls isStub: true.
	
	^ cls ! !

!MooseMonticelloImporter methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 21:03'!
ensureGlobalVariable: name value: value 
	 
	^ self globals 
		at: name 
		ifAbsent: [self createGlobalVariable: name value: value]! !

!MooseMonticelloImporter methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 20:48'!
ensureUnknownVariable: name
	^ self globals
		at: name
		ifAbsent: [self createUnknownVariable: name]
! !

!MooseMonticelloImporter methodsFor: 'public' stamp: 'Alexandre.Bergel 5/1/2009 21:40'!
fileNames
	^ self repository readableFileNames! !

!MooseMonticelloImporter methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 20:48'!
globals
	^ globals ifNil: [ globals := Dictionary new ]! !

!MooseMonticelloImporter methodsFor: 'public' stamp: 'Alexandre.Bergel 5/2/2009 22:15'!
import
	"Import all files found in the repository"
	| names |
	self setRepository.
	self setMooseModel.
	names := self fileNames.
	names do: [ :name | self importFileNamed: name ].
	^ referenceModel! !

!MooseMonticelloImporter methodsFor: 'public' stamp: 'jl 9/20/2010 16:15'!
importFileNamed: name 
	| version definitions visitor progressBar step |
	self setMooseModel.
	
	[ version := self repository versionFromFileNamed: name. ]
		on: Error 
		do: [:ex | self inform: 'Package ', name, ' not accessible from the file system?'. ^ self inModel].
		
	definitions := version snapshot definitions.
	
	
	MooseCustomTask runWithProgress: [:bar |
		visitor := MooseMonticelloClassPopulator new importer: self.
		definitions do: [ :def | def accept: visitor.
			bar increment].
	] description: 'Importing classes' length: (definitions size).

	self setSuperclassesAndCreateStubs.

	"create a scope, this is necessary for the method visitor which is VW aware"
	scope := Dictionary new.
	self inModel allClasses do: [:famixClass | 
		famixClass attributes do: [:famixAttribute | 
			(self scopeOfClass: famixClass) at: famixAttribute name bind: famixAttribute]].


	MooseCustomTask runWithProgress: [:bar |
		"import methods"
		definitions do: [ :def | 
			(def isKindOf: MCMethodDefinition)
			ifTrue: [
				visitor := MooseMonticelloMethodPopulator new importer: self.
				def accept: visitor.
				self inModel addAll: visitor entities]].
			bar increment.
		] description: 'Importing methods' length: (definitions size).


	^ self inModel! !

!MooseMonticelloImporter methodsFor: 'public' stamp: 'Alexandre.Bergel 5/4/2009 16:18'!
importLastVersionOf: projectName 
	^ self importFileNamed: (self lastVersionOf: projectName)! !

!MooseMonticelloImporter methodsFor: 'initialize-release' stamp: 'Alexandre.Bergel 6/25/2009 11:57'!
initialize
	super initialize.
	importingContext := ImportingContext new importMaximum; mergeClassAndMetaclass; yourself.! !

!MooseMonticelloImporter methodsFor: 'public' stamp: 'Alexandre.Bergel 5/4/2009 16:18'!
lastVersionOf: projectName 
	| versions |
	versions := self fileNames select: [ :f | f beginsWith: projectName , '-' ].
	^ (versions asSortedCollection: 
		[ :v1 :v2 | 
		(v1 
			copyFrom: (v1 indexOf: $.) + 1
			to: (v1 lastIndexOf: $.)) asInteger < (v1 
				copyFrom: (v1 indexOf: $.) + 1
				to: (v1 lastIndexOf: $.)) asInteger ]) last! !

!MooseMonticelloImporter methodsFor: 'public' stamp: 'Alexandre.Bergel 5/1/2009 09:45'!
repository
	^ repositoryCache ifNil: [ self setRepository. ^ repositoryCache ]! !

!MooseMonticelloImporter methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 20:06'!
scope
	^ scope! !

!MooseMonticelloImporter methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/4/2009 15:51'!
scopeOfClass: aClass 
	 
	aClass == nil ifTrue: [^nil]. 
	^scope
		at: aClass 
		ifAbsentPut: 
			[SmalltalkScope extend: (self scopeOfClass: aClass superclass)]! !

!MooseMonticelloImporter methodsFor: 'public' stamp: 'Alexandre.Bergel 5/2/2009 22:15'!
setMooseModel
	self inModel: MooseModel new.
	self inModel sourceLanguage: #SmalltalkMonticello! !

!MooseMonticelloImporter methodsFor: 'hook - private' stamp: 'Alexandre.Bergel 5/1/2009 13:05'!
setRepository
	"Must be called before importing"
	self subclassResponsibility! !

!MooseMonticelloImporter methodsFor: 'private' stamp: 'Alexandre.Bergel 5/1/2009 13:07'!
setSuperclassesAndCreateStubs
	| famixInheritance superclass |
	self inModel allClasses do: 
		[ :famixClass | 
		superclass := self ensureClassNamed: (famixClass propertyNamed: #superclassName).
		famixInheritance := FAMIXInheritance new
			subclass: famixClass;
			superclass: superclass.
		famixClass addSuperInheritance: famixInheritance.
		superclass addSubInheritance: famixInheritance ]! !

Object subclass: #MooseMonticelloVisitor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

MooseMonticelloVisitor subclass: #MooseMonticelloPopulator
	instanceVariableNames: 'importer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

MooseMonticelloPopulator subclass: #MooseMonticelloClassPopulator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

!MooseMonticelloClassPopulator methodsFor: 'accessing' stamp: 'Alexandre.Bergel 5/1/2009 11:58'!
visitClassDefinition: def
	importer 
		createClassNamed: def className asSymbol
		ivs: (def variables collect: [:each | each name ])
		superclassName: def superclassName! !

MooseMonticelloPopulator subclass: #MooseMonticelloMethodPopulator
	instanceVariableNames: 'entities scope'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moose-MonticelloImporter'!

!MooseMonticelloMethodPopulator methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/2/2009 19:45'!
addEntity: entity 
	entities add: entity! !

!MooseMonticelloMethodPopulator methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 21:02'!
createGlobalVariable: name value: value 
	 
	| global | 
	global := self globals 
		at: name 
		put: FAMIXGlobalVariable new. 
	global name: name. 
	Smalltalk at: name ifPresent: [:each | global declaredType: each class mooseName].
	^global! !

!MooseMonticelloMethodPopulator methodsFor: 'compatibility with visitor' stamp: 'tg 5/17/2009 16:53'!
createSpecialVariable: name forClass: aClass

	| attribute |
	attribute := (self scopeOfClass: aClass) at: name bind: FAMIXImplicitVariable new.
	attribute name: name.
	attribute container: aClass.	
	^attribute! !

!MooseMonticelloMethodPopulator methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 20:52'!
ensureClass: aClass 


	^ importer ensureClassNamed: aClass name asSymbol! !

!MooseMonticelloMethodPopulator methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 21:03'!
ensureGlobalVariable: name value: value 
	 
	^ importer ensureGlobalVariable: name value: value ! !

!MooseMonticelloMethodPopulator methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/2/2009 20:03'!
ensureImplicitVariable: name inClass: aClass
	
	^(self scopeOfClass: aClass)
		at: name
		ifAbsent:
			[self
				createSpecialVariable: name
				forClass: aClass]! !

!MooseMonticelloMethodPopulator methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/4/2009 16:22'!
ensureNamespace: aName
	^ nil! !

!MooseMonticelloMethodPopulator methodsFor: 'compatibility with visitor' stamp: 'Alexandre.Bergel 5/2/2009 20:48'!
ensureUnknownVariable: name
	^ importer ensureUnknownVariable: name
	
"	^globals
		at: name
		ifAbsent: [self createUnknownVariable: name]
"! !

!MooseMonticelloMethodPopulator methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/2/2009 21:39'!
entities
	^ entities! !

!MooseMonticelloMethodPopulator methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/2/2009 19:45'!
initialize
	super initialize.
	entities := OrderedCollection new! !

!MooseMonticelloMethodPopulator methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/4/2009 16:01'!
scopeOfClass: aClass
	^ importer scopeOfClass: aClass! !

!MooseMonticelloMethodPopulator methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 5/2/2009 18:57'!
setInfoOn: famixMethod withSource: sourceAsString
	| visitor |
	visitor := SmalltalkMethodVisitor on: self.
	visitor 
			runWithSource: sourceAsString
			and: famixMethod ! !

!MooseMonticelloMethodPopulator methodsFor: 'as yet unclassified' stamp: 'simon.denier 9/21/2009 14:47'!
visitMethodDefinition: definition 
	| famixMethod classOnWhichItIsDefined |
	famixMethod := FAMIXMethod new.
	famixMethod name: definition selector.
	famixMethod signature: definition selector.
	famixMethod hasClassScope: definition classIsMeta.
	"note that maybe the class does not exist yet, this happens if the method is an extension"
	classOnWhichItIsDefined := importer ensureClassNamed: definition className.
	classOnWhichItIsDefined addMethod: famixMethod.
	famixMethod parentType: classOnWhichItIsDefined.
	famixMethod category: definition category asSymbol.
	famixMethod isPublic: true.
	famixMethod sourceAnchor: (FAMIXSourceTextAnchor new source: definition source).
	famixMethod 
		propertyNamed: #timestamp
		put: definition timeStamp.
	importer inModel add: famixMethod.
	self 
		setInfoOn: famixMethod
		withSource: definition source.
	^ famixMethod! !

!MooseMonticelloPopulator methodsFor: 'accessing' stamp: 'Alexandre.Bergel 5/1/2009 11:31'!
importer
	^ importer! !

!MooseMonticelloPopulator methodsFor: 'accessing' stamp: 'Alexandre.Bergel 5/1/2009 11:31'!
importer: anObject
	importer := anObject! !

!MooseMonticelloPopulator methodsFor: 'accessing' stamp: 'Alexandre.Bergel 5/2/2009 19:07'!
importingContext
	^ importer importingContext! !

!MooseMonticelloVisitor methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:42'!
visitClassDefinition: def! !

!MooseMonticelloVisitor methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:42'!
visitClassTraitDefinition: definition! !

!MooseMonticelloVisitor methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:43'!
visitMetaclassDefinition: definition! !

!MooseMonticelloVisitor methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:43'!
visitMethodDefinition: definition! !

!MooseMonticelloVisitor methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:43'!
visitOrganizationDefinition: defintion! !

!MooseMonticelloVisitor methodsFor: 'as yet unclassified' stamp: 'Alexandre.Bergel 4/30/2009 23:43'!
visitTraitDefinition: definition! !
